/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviapartsviewhistorypopup.h"
#include <taviatabbutton.h>
#include <qcursor.h>
#include <qbitmap.h>
#include <qpainter.h>

TaviaPartsViewHistoryPopupItem::TaviaPartsViewHistoryPopupItem(QWidget *parent,int entry,const QString &caption,const QString &url,const QPixmap &thumbnail, const QPixmap &defaultPix,const QPixmap &activePix, const char *name)
 : QWidget(parent, name ,0)
{
    TaviaPartsViewHistoryPopupItem::defaultPix = defaultPix;
    TaviaPartsViewHistoryPopupItem::activePix = activePix;
    TaviaPartsViewHistoryPopupItem::title = caption;
    TaviaPartsViewHistoryPopupItem::url = url;
    historyEntry = entry;
    pressed = false;
    hover = false;
    description = new QLabel(0L,"description",WStyle_StaysOnTop | WStyle_Customize | WStyle_NoBorder | WStyle_Tool | WX11BypassWM);
    description->setFrameStyle(QFrame::Plain | QFrame::Box);
    description->hide();
    description->setText(caption + "\n" + url);
    setMouseTracking(true);
    setBackgroundPixmap(defaultPix);
    QGridLayout *glay = new QGridLayout(this,1,1,10);
    label = new QLabel(this);
    label->setFixedHeight(100);
    label->setFixedWidth(100);
    label->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    label->setPixmap(thumbnail);
    label->setMouseTracking(true);
    glay->addWidget(label,1,1);
}


TaviaPartsViewHistoryPopupItem::~TaviaPartsViewHistoryPopupItem()
{
    delete description;
}



/*!
    \fn TaviaPartsViewHistoryPopupItem::setPixmap(const QPixmap &pix)
 */
void TaviaPartsViewHistoryPopupItem::setPixmap(const QPixmap &pix)
{
    label->setPixmap(pix);
}

/*!
    \fn TaviaPartsViewHistoryPopup::mouseReleaseEvent(QMouseEvent *e)
 */
void TaviaPartsViewHistoryPopup::mouseReleaseEvent(QMouseEvent *e)
{
    emit historyEnter(-1);
}



/*!
    \fn TaviaPartsViewHistoryPopupItem::mouseMoveEvent(QMouseEvent *e)
 */
void TaviaPartsViewHistoryPopupItem::mouseMoveEvent(QMouseEvent *e)
{
    if (!hover) {
        hover = true;
        kdDebug() << "onMouseOver." << endl;
        setBackgroundPixmap(activePix);
        hoverCheck();
    } else {
        description->move(e->globalPos().x() + 10,e->globalPos().y() + 10);
        description->show();
    }
}


/*!
    \fn TaviaPartsViewHistoryPopupItem::mousePressEvent(QMouseEvent *e)
 */
void TaviaPartsViewHistoryPopupItem::mousePressEvent(QMouseEvent *e)
{
    pressed = true;
}


/*!
    \fn TaviaPartsViewHistoryPopupItem::mouseReleaseEvent(QMouseEvent *e)
 */
void TaviaPartsViewHistoryPopupItem::mouseReleaseEvent(QMouseEvent *e)
{
    if (pressed) emit clicked(historyEntry);
}


/*!
    \fn TaviaPartsViewHistoryPopupItem::hoverCheck()
 */
void TaviaPartsViewHistoryPopupItem::hoverCheck()
{
    if (!hasMouse()) {
        setBackgroundPixmap(defaultPix);
        pressed = false;
        hover = false;
        description->hide();
        return;
    } else QTimer::singleShot(10,this,SLOT(hoverCheck()));
}



TaviaPartsViewHistoryPopup::TaviaPartsViewHistoryPopup(TaviaPartsView *parent, const char *name,WFlags f)
 : QWidget(0L, name, f)
{    
    hide();
    partView = parent;
    thumbnail.setAutoDelete(true);
    mPos = QCursor::pos();
    QColor cF,cB,cA,cC;
    cF = QColor(255,255,255);
    cB = QColor(100,100,100);
    cA = QColor(255,10,10);
    cC = QColor(160,10,10);
    QBitmap shape;
    QPixmap defaultPix = TaviaTabButton::drawEdgePixmap(cF,cB,120,10,true,2,&shape);
    QPixmap currentPix = TaviaTabButton::drawEdgePixmap(cF,cC,120,10,true,2);
    QPixmap activePix = TaviaTabButton::drawEdgePixmap(cF,cA,120,10,true,2);

    setMouseTracking(true);
    
    QVBoxLayout *vlay = new QVBoxLayout(this);
    
    unsigned int c = (partView->currentHistory < 5 ? 0 : partView->currentHistory - 4);
    unsigned int m = partView->history.count();
    if (partView->currentHistory + 5 < partView->history.count()) m = partView->currentHistory + 4;
    int current = -1;
    unsigned int bCount = 0;
    for(;c < m;c ++) {
        TaviaPartsViewHistoryPopupItem *l;
        QImage img = partView->history.at(c)->thumbnail.convertToImage();
        QPixmap pix;
        pix.convertFromImage(img.smoothScale(100,100));
        if (c == partView->currentHistory) {
            l = new TaviaPartsViewHistoryPopupItem(this,c,partView->history.at(c)->caption,partView->history.at(c)->url.url(),pix,currentPix,activePix);
            current = thumbnail.count();
        } else l = new TaviaPartsViewHistoryPopupItem(this,c,partView->history.at(c)->caption,partView->history.at(c)->url.url(),pix,defaultPix,activePix);
        thumbnail.append(l);
        vlay->addWidget(l);
        connect(l,SIGNAL(clicked(int )),partView,SLOT(historyPopupEnter(int )));
        bCount ++;
    }
    QPainter paint;
    QBitmap mask;
    mask.resize(120,120 * bCount);
    paint.begin(&mask);
    for(unsigned int i = 0;i < bCount;i ++) {
        paint.drawPixmap(0,120 * i,shape);
    }
    paint.end();
    setAutoMask(true);
    setMask(mask);
    if (current == -1) current = thumbnail.count() - 1;
    QPoint pos = QCursor::pos();
    pos.setX(pos.x() - 60);
    pos.setY(pos.y() - (120 * current) - 60);
    move(pos);
    show();
    //grabMouse();
    //grabKeyboard();
}


TaviaPartsViewHistoryPopup::~TaviaPartsViewHistoryPopup()
{
    //releaseMouse();
    //releaseKeyboard();

}


/*!
    \fn TaviaPartsViewHistoryPopup::hideEvent(QHideEvent *e)
 */
void TaviaPartsViewHistoryPopup::hideEvent(QHideEvent *e)
{
    partView->historyPopupEnter(-1);
}


/*!
    \fn TaviaPartsViewHistoryPopup::wheelEvent(QWheelEvent *e)
 */
void TaviaPartsViewHistoryPopup::wheelEvent(QWheelEvent *e)
{
    if (e->delta() >= 0) {
        if (((pos().y() + 120) < QCursor::pos().y()) && ((pos().y() + 120) < mPos.y())) move(pos().x(),pos().y() + 120);
    } else {
        if (((pos().y() - 120 + height()) > QCursor::pos().y()) && ((pos().y() - 120 + height()) > mPos.y())) move(pos().x(),pos().y() - 120);
    }
        
}
