/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaurclistitem.h"
#include <kdebug.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <qlayout.h>
#include <qfont.h>
#include <qtimer.h>

TaviaURCListItem::TaviaURCListItem(TaviaURCObject *conn,QWidget *parent, const char *name)
 : QWidget(parent, name)
{
    pressed = false;
    isHover = false;
    setMouseTracking(true);
    to = conn;
    QHBoxLayout *hlay = new QHBoxLayout(this);
    icon = new QLabel(this);
    icon->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
    icon->setMouseTracking(true);
    label = new QLabel(this);
    label->setMouseTracking(true);
    label->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Minimum);
    //if (conn) label->setText(conn->getId());
    //else label->setText(i18n("New Connection"));
    
    dcButton = new QPushButton(this);
    dcButton->setText(i18n("disconnect"));
    if (!conn) dcButton->hide();
    connect(dcButton,SIGNAL(clicked()),this,SLOT(dcButtonClicked()));
    hlay->addWidget(icon);
    hlay->addWidget(label);
    hlay->addWidget(dcButton);
    
    
}


TaviaURCListItem::~TaviaURCListItem()
{
    kdDebug() << "URCListItem delete..." << endl;
}



/*!
    \fn TaviaURCListItem::mousePressEvent(QMouseEvent *e)
 */
void TaviaURCListItem::mousePressEvent(QMouseEvent *e)
{
    //if (e->button() == Qt::RightButton) emit dragStart(to);
    if (e->button() == Qt::LeftButton) pressed = true;
}


/*!
    \fn TaviaURCListItem::mouseReleaseEvent(QMouseEvent *e)
 */
void TaviaURCListItem::mouseReleaseEvent(QMouseEvent *e)
{
    if (e->button() == Qt::LeftButton) pressed = false;
}


/*!
    \fn TaviaURCListItem::mouseMoveEvent(QMouseEvent *e)
 */
void TaviaURCListItem::mouseMoveEvent(QMouseEvent *e)
{
    if (!isHover) {
        isHover = true;
        QFont f = label->font();
        f.setUnderline(true);
        label->setFont(f);
        hoverCheck();
    }
}


/*!
    \fn TaviaURCListItem::hoverCheck()
 */
void TaviaURCListItem::hoverCheck()
{
    if (hasMouse()) {
        QTimer::singleShot(10,this,SLOT(hoverCheck()));
    } else {
        isHover = false;
        QFont f = label->font();
        f.setUnderline(false);
        label->setFont(f);
        if (pressed) {
            emit dragStart(to);
            return;
        }
    }
}


/*!
    \fn TaviaURCListItem::setLabelText(const QString &text)
 */
void TaviaURCListItem::setLabelText(const QString &text)
{
    label->setText(text);
}


/*!
    \fn TaviaURCListItem::setIcon(const QString& icon)
 */
void TaviaURCListItem::setIcon(const QString& icon)
{
    TaviaURCListItem::icon->setPixmap(KGlobal::instance()->iconLoader()->loadIcon(icon,KIcon::Panel,KIcon::SizeSmall));
}


/*!
    \fn TaviaURCListItem::dcButtonClicked()
 */
void TaviaURCListItem::dcButtonClicked()
{
    emit dcButtonClicked(to);
}

