/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaurcmanager.h"
#include <qvaluelist.h>
#include <qdatetime.h>
#include <kdebug.h>

TaviaURCManager::TaviaURCManager(QObject *parent, const char *name)
 : QObject(parent, name)
{
}


TaviaURCManager::~TaviaURCManager()
{
}



/*!
    \fn TaviaURCManager::addURC(TaviaURCObject *conn)
 */
void TaviaURCManager::addURC(TaviaURCObject *conn)
{
    if (connection.contains(conn)) return;
    connection[conn] = QPtrList<TaviaURCObject>();
    QString id;
    id = QString::number((QDate::fromString("1970-01-01",Qt::ISODate).daysTo(QDate::currentDate()) * 86400) +
        QTime::fromString("00:00:00",Qt::ISODate).secsTo(QTime::currentTime())) +
        QString::number(QTime::currentTime().msec());
    conn->id = id;
    connect(conn,SIGNAL(request(const KURL&, const KParts::URLArgs& )),this,SLOT(gate(const KURL&, const KParts::URLArgs& )));
    kdDebug() << "addURC : " << conn << endl;
    return;
}


/*!
    \fn TaviaURCManager::delURC(TaviaURCObject *conn)
 */
void TaviaURCManager::delURC(TaviaURCObject *conn)
{
    if (!connection.contains(conn)) return;
    // ݽ֤ʡ
    QValueList<TaviaURCObject *> l = connection.keys();
    for (unsigned int i = 0;i < l.count();i ++) {
        for(unsigned int j = 0;j < connection[l[i]].count();j ++) {
            if (connection[l[i]].at(j) == conn) {
                connection[l[i]].remove(j);
            }
        }
    }
    connection.remove(conn);
    kdDebug() << "delURC : " << conn << endl;
    return;
}


/*!
    \fn TaviaURCManager::addConnection(TaviaURCObject *from,TaviaURCObject *to)
 */
void TaviaURCManager::addConnection(TaviaURCObject *from,TaviaURCObject *to)
{
    if ((!connection.contains(from)) || (!connection.contains(to))) {
        kdDebug() << "URCObject not found..." << endl;
        return;
    }
    if (connection[from].find(to) == -1) connection[from].append(to);
    kdDebug() << "URC+ : " << from << " -> " << to << endl;
    return;
}


/*!
    \fn TaviaURCManager::delConnection(TaviaURCObject *from,TaviaURCObject *to)
 */
void TaviaURCManager::delConnection(TaviaURCObject *from,TaviaURCObject *to)
{
    if ((!connection.contains(from)) || (!connection.contains(to))) return;
    int c = connection[from].find(to);
    if (c != -1) connection[from].remove(c);
    kdDebug() << "URC- : " << from << " -> " << to << endl;
    return;
}


/*!
    \fn TaviaURCManager::gate(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaURCManager::gate(const KURL &url,const KParts::URLArgs &args)
{
    TaviaURCObject *requestor = dynamic_cast<TaviaURCObject *>(const_cast<QObject *>(sender()));
    if ((!requestor) || (!connection.contains(requestor))) return;
    for(unsigned int i = 0;i < connection[requestor].count();i ++) {
        emit connection[requestor].at(i)->receive(url,args);
    }
}


/*!
    \fn TaviaURCManager::getConnectionList(TaviaURCObject *o)
 */
QPtrList<TaviaURCObject> TaviaURCManager::getConnectionList(TaviaURCObject *o)
{
    QPtrList<TaviaURCObject> l;
    if (connection.contains(o)) l = connection[o];
    return(l);
}


/*!
    \fn TaviaURCManager::getURCObjectList()
 */
QPtrList<TaviaURCObject> TaviaURCManager::getURCObjectList()
{
    QValueList<TaviaURCObject *> l = connection.keys();
    QPtrList<TaviaURCObject> p;
    for(unsigned i = 0;i < l.count();i ++) {
        p.append(l[i]);
    }
    return(p);
}
