/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *   Copyright (C) 2004 by kzk(Kazuki Ohta)                                *
 *   mover@hct.zaq.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaurlbar.h"

#include <qtoolbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qstring.h>
#include <qstringlist.h>

#include <kurl.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kcombobox.h>
#include <kiconloader.h>
#include <kcompletionbox.h>

TaviaURLBar::TaviaURLBar(QWidget *parent, const char *name, WFlags f)
    : QWidget(parent, name, f)
{
    if (!mCompletion) {
        mCompletion = new KCompletion;
        mCompletion->setOrder(KCompletion::Sorted);
        kdDebug() << "Created KCompletion." << endl;
    }
    connect(mCompletion, SIGNAL(match(const QString&)),
             this, SLOT(slotMatch(const QString&)));

    uCombo = new KComboBox(this,"urlCombo");
    uCombo->setEditable(true);
    uCombo->setContextMenuEnabled(true);
    uCombo->setURLDropsEnabled(true);
    uCombo->setInsertionPolicy(QComboBox::NoInsertion);
    uCombo->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
    uCombo->setCompletionObject(mCompletion, false);
    uCombo->setAutoDeleteCompletionObject(false);
    uCombo->setCompletionMode(mCompletion->completionMode());

    connect(uCombo,SIGNAL(completionModeChanged(KGlobalSettings::Completion)),
            this,SLOT(slotCompletionModeChanged(KGlobalSettings::Completion)));
    connect(uCombo,SIGNAL(completion(const QString&)),
            this,SLOT(slotMakeCompletion(const QString&)));
    connect(uCombo,SIGNAL(activated(const QString&)),
            this,SLOT(slotActivated(const QString&)));
    connect(uCombo,SIGNAL(returnPressed()),this,SLOT(slotReturnPressed()));
    
    connect(uCombo,SIGNAL(completion(const QString&)),this,SLOT(completionConnect()));

    urlEnter = new QToolButton(this,"urlEnter");
    urlEnter->setAutoRaise(true);
    urlEnter->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("key_enter",KIcon::Toolbar,KIcon::SizeSmall));

    urlErase = new QToolButton(this,"urlErase");
    urlErase->setAutoRaise(true);
    urlErase->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("locationbar_erase",KIcon::Toolbar,KIcon::SizeSmall));

    QLabel *label = new QLabel(this,"label");
    label->setText("URL: ");
    QHBoxLayout *lay = new QHBoxLayout(this);
    lay->addWidget(urlErase);
    lay->addWidget(label);
    lay->addWidget(uCombo);
    lay->addWidget(urlEnter);

    connect(urlErase,SIGNAL(clicked()),this,SLOT(clearURLBar()));
    connect(urlEnter,SIGNAL(clicked()),this,SLOT(slotReturnPressed()));
}


TaviaURLBar::~TaviaURLBar()
{
    //if (mCompletion)
    //    delete mCompletion;
}

KCompletion * TaviaURLBar::mCompletion = 0L;

void TaviaURLBar::setEditURL(const KURL& url)
{
    uCombo->setEditURL(url);
}

/*!
    \fn TaviaURLBar::setFocus()
 */
void TaviaURLBar::setFocus()
{
    uCombo->setFocus();
}

/*!
    \fn TaviaURLBar::clearURLBar()
 */
void TaviaURLBar::clearURLBar()
{
    uCombo->setFocus();
    uCombo->setCurrentText("");
}

/*!
    \fn TaviaURLBar::returnPressed()
 */
void TaviaURLBar::slotReturnPressed()
{
    emit enterURL(KURL(uCombo->currentText()));
}

/*!
    \fn TaviaURLBar::addURLHistory(const KURL &url)
 */
void TaviaURLBar::addURLHistory(const KURL &url)
{
    // ʣɤ
    if(uCombo->contains(url.url()))
        return;

    uCombo->insertItem(url.url());
    uCombo->setCurrentItem(uCombo->count() - 1);

    mCompletion->addItem(url.url());
}

/*!
    \fn TaviaURLBar::slotEnterURL(const KURL &url)
 */
void TaviaURLBar::slotEnterURL(const KURL &url)
{
    emit enterURL(url);
}

/*!
    \fn TaviaURLBar::setEditText(const QString &text)
 */
void TaviaURLBar::setEditText(const QString &text)
{
    uCombo->setEditText(text);
}

/*!
    \fn TaviaURLBar::setEnabled(bool b)
 */
void TaviaURLBar::setEnabled(bool b)
{
    // ޤ̵̣
    if (!b) {
        uCombo->clearFocus();
    }
    QWidget::setEnabled(b);
}

/*!
    \fn TaviaURLBar::slotCompletionModeChanged(KGlobalSettings::Completion m)
 */
void TaviaURLBar::slotCompletionModeChanged(KGlobalSettings::Completion m)
{
    mCompletion->setCompletionMode(m);

    KConfig *config = KGlobal::config();
    config->setGroup("Settings");
    config->writeEntry("CompletionMode", (int)uCombo->completionMode());
    config->sync();

    /*
     * TODO : ¾PartsViewURLBarˤ
     */
}

/*!
    \fn TaviaURLBar::slotMakeCompletion(const QString &str)
 */
void TaviaURLBar::slotMakeCompletion(const QString &str)
{
    /*
    kdDebug() << "TaviaURLBar::slotMakeCompletion : str = " << str << endl
              << "items = " << mCompletion->items() << endl;
    */

    QString completion = mCompletion->makeCompletion(str);
     if ( uCombo->completionMode() == KGlobalSettings::CompletionPopup ||
          uCombo->completionMode() == KGlobalSettings::CompletionPopupAuto )
         uCombo->setCompletedItems(checkPrefixes(str));
}

/*!
    \fn TaviaURLBar::slotMatch(const QString &str)
 */
void TaviaURLBar::slotMatch(const QString &str)
{
//    kdDebug() << "TaviaURLBar::slotMatch" << endl;

    if (str.isEmpty())
        return;

    if (uCombo->completionMode() == KGlobalSettings::CompletionPopup ||
         uCombo->completionMode() == KGlobalSettings::CompletionPopupAuto) {
        QStringList items = mCompletion->allMatches();
        items += checkPrefixes(uCombo->currentText());

        uCombo->setCompletedItems(items);
    }
}

/*!
    \fn TaviaURLBar::checkPrefixes(const QString &s)
 */
QStringList TaviaURLBar::checkPrefixes(const QString &s)
{
    if(s.isEmpty())
    return QStringList();

    QString http = "http://";
    QString https = "https://";
    QString www = "http://www.";
    QString wwws = "https://www.";
    QString ftp = "ftp://";
    QString ftpftp = "ftp://ftp.";
    QStringList matches = mCompletion->allMatches(s);

    if (!s.startsWith(ftp))
        matches += mCompletion->allMatches(ftp + s);
    if (!s.startsWith(https))
        matches += mCompletion->allMatches(https + s);
    if (!s.startsWith(http))
        matches += mCompletion->allMatches(http + s);
    if (!s.startsWith(www))
        matches += mCompletion->allMatches(www + s);
    if (!s.startsWith(wwws))
        matches += mCompletion->allMatches(wwws + s);
    if (!s.startsWith(ftpftp))
        matches += mCompletion->allMatches(ftpftp + s);

    return matches;
}

/*!
    \fn TaviaURLBar::slotActivated(const QString& str)
 */
void TaviaURLBar::slotActivated(const QString& str)
{
    kdDebug() << "activated : str = " << str << endl;

    uCombo->setEditText(str);
    slotEnterURL(KURL(str));
}


/*!
    \fn TaviaURLBar::slotHighlighted(const QString& str)
 */
void TaviaURLBar::slotHighlighted(const QString& str)
{
    kdDebug() << "highlighted : str = " << str << endl;

    uCombo->setEditText(str);
}


/*!
    \fn TaviaURLBar::completionConnect()
 */
void TaviaURLBar::completionConnect()
{
    if (uCombo->completionBox(false)) {
        disconnect(uCombo,SIGNAL(completion(const QString&)),this,SLOT(completionConnect()));
        connect(uCombo->completionBox(),SIGNAL(highlighted(const QString&)),
                this,SLOT(slotHighlighted(const QString&)));
        connect(uCombo->completionBox(),SIGNAL(activated(const QString&)),
                this,SLOT(slotActivated(const QString&)));
        kdDebug() << "completionBox Connected." << endl;
    }
}
